@@ -10,7 +10,13 @@
 // Package home: https://github.com/klauspost/cpuid
 package cpuid
 
-import "strings"
+import (
+	"math"
+	"strings"
+)
+
+// AMD refererence: https://www.amd.com/system/files/TechDocs/25481.pdf
+// and Processor Programming Reference (PPR)
 
 // Vendor is a representation of a CPU vendor.
 type Vendor int
@@ -31,55 +37,66 @@ const (
 )
 
 const (
-	CMOV        = 1 << iota // i686 CMOV
-	NX                      // NX (No-Execute) bit
-	AMD3DNOW                // AMD 3DNOW
-	AMD3DNOWEXT             // AMD 3DNowExt
-	MMX                     // standard MMX
-	MMXEXT                  // SSE integer functions or AMD MMX ext
-	SSE                     // SSE functions
-	SSE2                    // P4 SSE functions
-	SSE3                    // Prescott SSE3 functions
-	SSSE3                   // Conroe SSSE3 functions
-	SSE4                    // Penryn SSE4.1 functions
-	SSE4A                   // AMD Barcelona microarchitecture SSE4a instructions
-	SSE42                   // Nehalem SSE4.2 functions
-	AVX                     // AVX functions
-	AVX2                    // AVX2 functions
-	FMA3                    // Intel FMA 3
-	FMA4                    // Bulldozer FMA4 functions
-	XOP                     // Bulldozer XOP functions
-	F16C                    // Half-precision floating-point conversion
-	BMI1                    // Bit Manipulation Instruction Set 1
-	BMI2                    // Bit Manipulation Instruction Set 2
-	TBM                     // AMD Trailing Bit Manipulation
-	LZCNT                   // LZCNT instruction
-	POPCNT                  // POPCNT instruction
-	AESNI                   // Advanced Encryption Standard New Instructions
-	CLMUL                   // Carry-less Multiplication
-	HTT                     // Hyperthreading (enabled)
-	HLE                     // Hardware Lock Elision
-	RTM                     // Restricted Transactional Memory
-	RDRAND                  // RDRAND instruction is available
-	RDSEED                  // RDSEED instruction is available
-	ADX                     // Intel ADX (Multi-Precision Add-Carry Instruction Extensions)
-	SHA                     // Intel SHA Extensions
-	AVX512F                 // AVX-512 Foundation
-	AVX512DQ                // AVX-512 Doubleword and Quadword Instructions
-	AVX512IFMA              // AVX-512 Integer Fused Multiply-Add Instructions
-	AVX512PF                // AVX-512 Prefetch Instructions
-	AVX512ER                // AVX-512 Exponential and Reciprocal Instructions
-	AVX512CD                // AVX-512 Conflict Detection Instructions
-	AVX512BW                // AVX-512 Byte and Word Instructions
-	AVX512VL                // AVX-512 Vector Length Extensions
-	AVX512VBMI              // AVX-512 Vector Bit Manipulation Instructions
-	MPX                     // Intel MPX (Memory Protection Extensions)
-	ERMS                    // Enhanced REP MOVSB/STOSB
-	RDTSCP                  // RDTSCP Instruction
-	CX16                    // CMPXCHG16B Instruction
-	SGX                     // Software Guard Extensions
-	IBPB                    // Indirect Branch Restricted Speculation (IBRS) and Indirect Branch Predictor Barrier (IBPB)
-	STIBP                   // Single Thread Indirect Branch Predictors
+	CMOV               = 1 << iota // i686 CMOV
+	NX                             // NX (No-Execute) bit
+	AMD3DNOW                       // AMD 3DNOW
+	AMD3DNOWEXT                    // AMD 3DNowExt
+	MMX                            // standard MMX
+	MMXEXT                         // SSE integer functions or AMD MMX ext
+	SSE                            // SSE functions
+	SSE2                           // P4 SSE functions
+	SSE3                           // Prescott SSE3 functions
+	SSSE3                          // Conroe SSSE3 functions
+	SSE4                           // Penryn SSE4.1 functions
+	SSE4A                          // AMD Barcelona microarchitecture SSE4a instructions
+	SSE42                          // Nehalem SSE4.2 functions
+	AVX                            // AVX functions
+	AVX2                           // AVX2 functions
+	FMA3                           // Intel FMA 3
+	FMA4                           // Bulldozer FMA4 functions
+	XOP                            // Bulldozer XOP functions
+	F16C                           // Half-precision floating-point conversion
+	BMI1                           // Bit Manipulation Instruction Set 1
+	BMI2                           // Bit Manipulation Instruction Set 2
+	TBM                            // AMD Trailing Bit Manipulation
+	LZCNT                          // LZCNT instruction
+	POPCNT                         // POPCNT instruction
+	AESNI                          // Advanced Encryption Standard New Instructions
+	CLMUL                          // Carry-less Multiplication
+	HTT                            // Hyperthreading (enabled)
+	HLE                            // Hardware Lock Elision
+	RTM                            // Restricted Transactional Memory
+	RDRAND                         // RDRAND instruction is available
+	RDSEED                         // RDSEED instruction is available
+	ADX                            // Intel ADX (Multi-Precision Add-Carry Instruction Extensions)
+	SHA                            // Intel SHA Extensions
+	AVX512F                        // AVX-512 Foundation
+	AVX512DQ                       // AVX-512 Doubleword and Quadword Instructions
+	AVX512IFMA                     // AVX-512 Integer Fused Multiply-Add Instructions
+	AVX512PF                       // AVX-512 Prefetch Instructions
+	AVX512ER                       // AVX-512 Exponential and Reciprocal Instructions
+	AVX512CD                       // AVX-512 Conflict Detection Instructions
+	AVX512BW                       // AVX-512 Byte and Word Instructions
+	AVX512VL                       // AVX-512 Vector Length Extensions
+	AVX512VBMI                     // AVX-512 Vector Bit Manipulation Instructions
+	AVX512VBMI2                    // AVX-512 Vector Bit Manipulation Instructions, Version 2
+	AVX512VNNI                     // AVX-512 Vector Neural Network Instructions
+	AVX512VPOPCNTDQ                // AVX-512 Vector Population Count Doubleword and Quadword
+	GFNI                           // Galois Field New Instructions
+	VAES                           // Vector AES
+	AVX512BITALG                   // AVX-512 Bit Algorithms
+	VPCLMULQDQ                     // Carry-Less Multiplication Quadword
+	AVX512BF16                     // AVX-512 BFLOAT16 Instructions
+	AVX512VP2INTERSECT             // AVX-512 Intersect for D/Q
+	MPX                            // Intel MPX (Memory Protection Extensions)
+	ERMS                           // Enhanced REP MOVSB/STOSB
+	RDTSCP                         // RDTSCP Instruction
+	CX16                           // CMPXCHG16B Instruction
+	SGX                            // Software Guard Extensions
+	SGXLC                          // Software Guard Extensions Launch Control
+	IBPB                           // Indirect Branch Restricted Speculation (IBRS) and Indirect Branch Predictor Barrier (IBPB)
+	STIBP                          // Single Thread Indirect Branch Predictors
+	VMX                            // Virtual Machine Extensions
 
 	// Performance indicators
 	SSE2SLOW // SSE2 is supported, but usually not faster
@@ -88,55 +105,66 @@ const (
 )
 
 var flagNames = map[Flags]string{
-	CMOV:        "CMOV",        // i686 CMOV
-	NX:          "NX",          // NX (No-Execute) bit
-	AMD3DNOW:    "AMD3DNOW",    // AMD 3DNOW
-	AMD3DNOWEXT: "AMD3DNOWEXT", // AMD 3DNowExt
-	MMX:         "MMX",         // Standard MMX
-	MMXEXT:      "MMXEXT",      // SSE integer functions or AMD MMX ext
-	SSE:         "SSE",         // SSE functions
-	SSE2:        "SSE2",        // P4 SSE2 functions
-	SSE3:        "SSE3",        // Prescott SSE3 functions
-	SSSE3:       "SSSE3",       // Conroe SSSE3 functions
-	SSE4:        "SSE4.1",      // Penryn SSE4.1 functions
-	SSE4A:       "SSE4A",       // AMD Barcelona microarchitecture SSE4a instructions
-	SSE42:       "SSE4.2",      // Nehalem SSE4.2 functions
-	AVX:         "AVX",         // AVX functions
-	AVX2:        "AVX2",        // AVX functions
-	FMA3:        "FMA3",        // Intel FMA 3
-	FMA4:        "FMA4",        // Bulldozer FMA4 functions
-	XOP:         "XOP",         // Bulldozer XOP functions
-	F16C:        "F16C",        // Half-precision floating-point conversion
-	BMI1:        "BMI1",        // Bit Manipulation Instruction Set 1
-	BMI2:        "BMI2",        // Bit Manipulation Instruction Set 2
-	TBM:         "TBM",         // AMD Trailing Bit Manipulation
-	LZCNT:       "LZCNT",       // LZCNT instruction
-	POPCNT:      "POPCNT",      // POPCNT instruction
-	AESNI:       "AESNI",       // Advanced Encryption Standard New Instructions
-	CLMUL:       "CLMUL",       // Carry-less Multiplication
-	HTT:         "HTT",         // Hyperthreading (enabled)
-	HLE:         "HLE",         // Hardware Lock Elision
-	RTM:         "RTM",         // Restricted Transactional Memory
-	RDRAND:      "RDRAND",      // RDRAND instruction is available
-	RDSEED:      "RDSEED",      // RDSEED instruction is available
-	ADX:         "ADX",         // Intel ADX (Multi-Precision Add-Carry Instruction Extensions)
-	SHA:         "SHA",         // Intel SHA Extensions
-	AVX512F:     "AVX512F",     // AVX-512 Foundation
-	AVX512DQ:    "AVX512DQ",    // AVX-512 Doubleword and Quadword Instructions
-	AVX512IFMA:  "AVX512IFMA",  // AVX-512 Integer Fused Multiply-Add Instructions
-	AVX512PF:    "AVX512PF",    // AVX-512 Prefetch Instructions
-	AVX512ER:    "AVX512ER",    // AVX-512 Exponential and Reciprocal Instructions
-	AVX512CD:    "AVX512CD",    // AVX-512 Conflict Detection Instructions
-	AVX512BW:    "AVX512BW",    // AVX-512 Byte and Word Instructions
-	AVX512VL:    "AVX512VL",    // AVX-512 Vector Length Extensions
-	AVX512VBMI:  "AVX512VBMI",  // AVX-512 Vector Bit Manipulation Instructions
-	MPX:         "MPX",         // Intel MPX (Memory Protection Extensions)
-	ERMS:        "ERMS",        // Enhanced REP MOVSB/STOSB
-	RDTSCP:      "RDTSCP",      // RDTSCP Instruction
-	CX16:        "CX16",        // CMPXCHG16B Instruction
-	SGX:         "SGX",         // Software Guard Extensions
-	IBPB:        "IBPB",        // Indirect Branch Restricted Speculation and Indirect Branch Predictor Barrier
-	STIBP:       "STIBP",       // Single Thread Indirect Branch Predictors
+	CMOV:               "CMOV",               // i686 CMOV
+	NX:                 "NX",                 // NX (No-Execute) bit
+	AMD3DNOW:           "AMD3DNOW",           // AMD 3DNOW
+	AMD3DNOWEXT:        "AMD3DNOWEXT",        // AMD 3DNowExt
+	MMX:                "MMX",                // Standard MMX
+	MMXEXT:             "MMXEXT",             // SSE integer functions or AMD MMX ext
+	SSE:                "SSE",                // SSE functions
+	SSE2:               "SSE2",               // P4 SSE2 functions
+	SSE3:               "SSE3",               // Prescott SSE3 functions
+	SSSE3:              "SSSE3",              // Conroe SSSE3 functions
+	SSE4:               "SSE4.1",             // Penryn SSE4.1 functions
+	SSE4A:              "SSE4A",              // AMD Barcelona microarchitecture SSE4a instructions
+	SSE42:              "SSE4.2",             // Nehalem SSE4.2 functions
+	AVX:                "AVX",                // AVX functions
+	AVX2:               "AVX2",               // AVX functions
+	FMA3:               "FMA3",               // Intel FMA 3
+	FMA4:               "FMA4",               // Bulldozer FMA4 functions
+	XOP:                "XOP",                // Bulldozer XOP functions
+	F16C:               "F16C",               // Half-precision floating-point conversion
+	BMI1:               "BMI1",               // Bit Manipulation Instruction Set 1
+	BMI2:               "BMI2",               // Bit Manipulation Instruction Set 2
+	TBM:                "TBM",                // AMD Trailing Bit Manipulation
+	LZCNT:              "LZCNT",              // LZCNT instruction
+	POPCNT:             "POPCNT",             // POPCNT instruction
+	AESNI:              "AESNI",              // Advanced Encryption Standard New Instructions
+	CLMUL:              "CLMUL",              // Carry-less Multiplication
+	HTT:                "HTT",                // Hyperthreading (enabled)
+	HLE:                "HLE",                // Hardware Lock Elision
+	RTM:                "RTM",                // Restricted Transactional Memory
+	RDRAND:             "RDRAND",             // RDRAND instruction is available
+	RDSEED:             "RDSEED",             // RDSEED instruction is available
+	ADX:                "ADX",                // Intel ADX (Multi-Precision Add-Carry Instruction Extensions)
+	SHA:                "SHA",                // Intel SHA Extensions
+	AVX512F:            "AVX512F",            // AVX-512 Foundation
+	AVX512DQ:           "AVX512DQ",           // AVX-512 Doubleword and Quadword Instructions
+	AVX512IFMA:         "AVX512IFMA",         // AVX-512 Integer Fused Multiply-Add Instructions
+	AVX512PF:           "AVX512PF",           // AVX-512 Prefetch Instructions
+	AVX512ER:           "AVX512ER",           // AVX-512 Exponential and Reciprocal Instructions
+	AVX512CD:           "AVX512CD",           // AVX-512 Conflict Detection Instructions
+	AVX512BW:           "AVX512BW",           // AVX-512 Byte and Word Instructions
+	AVX512VL:           "AVX512VL",           // AVX-512 Vector Length Extensions
+	AVX512VBMI:         "AVX512VBMI",         // AVX-512 Vector Bit Manipulation Instructions
+	AVX512VBMI2:        "AVX512VBMI2",        // AVX-512 Vector Bit Manipulation Instructions, Version 2
+	AVX512VNNI:         "AVX512VNNI",         // AVX-512 Vector Neural Network Instructions
+	AVX512VPOPCNTDQ:    "AVX512VPOPCNTDQ",    // AVX-512 Vector Population Count Doubleword and Quadword
+	GFNI:               "GFNI",               // Galois Field New Instructions
+	VAES:               "VAES",               // Vector AES
+	AVX512BITALG:       "AVX512BITALG",       // AVX-512 Bit Algorithms
+	VPCLMULQDQ:         "VPCLMULQDQ",         // Carry-Less Multiplication Quadword
+	AVX512BF16:         "AVX512BF16",         // AVX-512 BFLOAT16 Instruction
+	AVX512VP2INTERSECT: "AVX512VP2INTERSECT", // AVX-512 Intersect for D/Q
+	MPX:                "MPX",                // Intel MPX (Memory Protection Extensions)
+	ERMS:               "ERMS",               // Enhanced REP MOVSB/STOSB
+	RDTSCP:             "RDTSCP",             // RDTSCP Instruction
+	CX16:               "CX16",               // CMPXCHG16B Instruction
+	SGX:                "SGX",                // Software Guard Extensions
+	SGXLC:              "SGXLC",              // Software Guard Extensions Launch Control
+	IBPB:               "IBPB",               // Indirect Branch Restricted Speculation and Indirect Branch Predictor Barrier
+	STIBP:              "STIBP",              // Single Thread Indirect Branch Predictors
+	VMX:                "VMX",                // Virtual Machine Extensions
 
 	// Performance indicators
 	SSE2SLOW: "SSE2SLOW", // SSE2 supported, but usually not faster
@@ -156,11 +184,12 @@ type CPUInfo struct {
 	Family         int    // CPU family number
 	Model          int    // CPU model number
 	CacheLine      int    // Cache line size in bytes. Will be 0 if undetectable.
+	Hz             int64  // Clock speed, if known
 	Cache          struct {
 		L1I int // L1 Instruction Cache (per core or shared). Will be -1 if undetected
 		L1D int // L1 Data Cache (per core or shared). Will be -1 if undetected
 		L2  int // L2 Cache (per core or shared). Will be -1 if undetected
-		L3  int // L3 Instruction Cache (per core or shared). Will be -1 if undetected
+		L3  int // L3 Cache (per core, per ccx or shared). Will be -1 if undetected
 	}
 	SGX       SGXSupport
 	maxFunc   uint32
@@ -198,11 +227,12 @@ func Detect() {
 	CPU.CacheLine = cacheLine()
 	CPU.Family, CPU.Model = familyModel()
 	CPU.Features = support()
-	CPU.SGX = hasSGX(CPU.Features&SGX != 0)
+	CPU.SGX = hasSGX(CPU.Features&SGX != 0, CPU.Features&SGXLC != 0)
 	CPU.ThreadsPerCore = threadsPerCore()
 	CPU.LogicalCores = logicalCores()
 	CPU.PhysicalCores = physicalCores()
 	CPU.VendorID = vendorID()
+	CPU.Hz = hertz(CPU.BrandName)
 	CPU.cacheSize()
 }
 
@@ -223,6 +253,11 @@ func (c CPUInfo) Amd3dnowExt() bool {
 	return c.Features&AMD3DNOWEXT != 0
 }
 
+// VMX indicates support of VMX
+func (c CPUInfo) VMX() bool {
+	return c.Features&VMX != 0
+}
+
 // MMX indicates support of MMX instructions
 func (c CPUInfo) MMX() bool {
 	return c.Features&MMX != 0
@@ -433,6 +468,51 @@ func (c CPUInfo) AVX512VBMI() bool {
 	return c.Features&AVX512VBMI != 0
 }
 
+// AVX512VBMI2 indicates support of AVX-512 Vector Bit Manipulation Instructions, Version 2
+func (c CPUInfo) AVX512VBMI2() bool {
+	return c.Features&AVX512VBMI2 != 0
+}
+
+// AVX512VNNI indicates support of AVX-512 Vector Neural Network Instructions
+func (c CPUInfo) AVX512VNNI() bool {
+	return c.Features&AVX512VNNI != 0
+}
+
+// AVX512VPOPCNTDQ indicates support of AVX-512 Vector Population Count Doubleword and Quadword
+func (c CPUInfo) AVX512VPOPCNTDQ() bool {
+	return c.Features&AVX512VPOPCNTDQ != 0
+}
+
+// GFNI indicates support of Galois Field New Instructions
+func (c CPUInfo) GFNI() bool {
+	return c.Features&GFNI != 0
+}
+
+// VAES indicates support of Vector AES
+func (c CPUInfo) VAES() bool {
+	return c.Features&VAES != 0
+}
+
+// AVX512BITALG indicates support of AVX-512 Bit Algorithms
+func (c CPUInfo) AVX512BITALG() bool {
+	return c.Features&AVX512BITALG != 0
+}
+
+// VPCLMULQDQ indicates support of Carry-Less Multiplication Quadword
+func (c CPUInfo) VPCLMULQDQ() bool {
+	return c.Features&VPCLMULQDQ != 0
+}
+
+// AVX512BF16 indicates support of
+func (c CPUInfo) AVX512BF16() bool {
+	return c.Features&AVX512BF16 != 0
+}
+
+// AVX512VP2INTERSECT indicates support of
+func (c CPUInfo) AVX512VP2INTERSECT() bool {
+	return c.Features&AVX512VP2INTERSECT != 0
+}
+
 // MPX indicates support of Intel MPX (Memory Protection Extensions)
 func (c CPUInfo) MPX() bool {
 	return c.Features&MPX != 0
@@ -529,6 +609,65 @@ func (c CPUInfo) LogicalCPU() int {
 	return int(ebx >> 24)
 }
 
+// hertz tries to compute the clock speed of the CPU. If leaf 15 is
+// supported, use it, otherwise parse the brand string. Yes, really.
+func hertz(model string) int64 {
+	mfi := maxFunctionID()
+	if mfi >= 0x15 {
+		eax, ebx, ecx, _ := cpuid(0x15)
+		if eax != 0 && ebx != 0 && ecx != 0 {
+			return int64((int64(ecx) * int64(ebx)) / int64(eax))
+		}
+	}
+	// computeHz determines the official rated speed of a CPU from its brand
+	// string. This insanity is *actually the official documented way to do
+	// this according to Intel*, prior to leaf 0x15 existing. The official
+	// documentation only shows this working for exactly `x.xx` or `xxxx`
+	// cases, e.g., `2.50GHz` or `1300MHz`; this parser will accept other
+	// sizes.
+	hz := strings.LastIndex(model, "Hz")
+	if hz < 3 {
+		return -1
+	}
+	var multiplier int64
+	switch model[hz-1] {
+	case 'M':
+		multiplier = 1000 * 1000
+	case 'G':
+		multiplier = 1000 * 1000 * 1000
+	case 'T':
+		multiplier = 1000 * 1000 * 1000 * 1000
+	}
+	if multiplier == 0 {
+		return -1
+	}
+	freq := int64(0)
+	divisor := int64(0)
+	decimalShift := int64(1)
+	var i int
+	for i = hz - 2; i >= 0 && model[i] != ' '; i-- {
+		if model[i] >= '0' && model[i] <= '9' {
+			freq += int64(model[i]-'0') * decimalShift
+			decimalShift *= 10
+		} else if model[i] == '.' {
+			if divisor != 0 {
+				return -1
+			}
+			divisor = decimalShift
+		} else {
+			return -1
+		}
+	}
+	// we didn't find a space
+	if i < 0 {
+		return -1
+	}
+	if divisor != 0 {
+		return (freq * multiplier) / divisor
+	}
+	return freq * multiplier
+}
+
 // VM Will return true if the cpu id indicates we are in
 // a virtual machine. This is only a hint, and will very likely
 // have many false negatives.
@@ -587,11 +726,14 @@ func brandName() string {
 
 func threadsPerCore() int {
 	mfi := maxFunctionID()
-	if mfi < 0x4 || vendorID() != Intel {
+	if mfi < 0x4 || (vendorID() != Intel && vendorID() != AMD) {
 		return 1
 	}
 
 	if mfi < 0xb {
+		if vendorID() != Intel {
+			return 1
+		}
 		_, b, _, d := cpuid(1)
 		if (d & (1 << 28)) != 0 {
 			// v will contain logical core count
@@ -655,6 +797,13 @@ func physicalCores() int {
 	case Intel:
 		return logicalCores() / threadsPerCore()
 	case AMD, Hygon:
+		lc := logicalCores()
+		tpc := threadsPerCore()
+		if lc > 0 && tpc > 0 {
+			return lc / tpc
+		}
+		// The following is inaccurate on AMD EPYC 7742 64-Core Processor
+
 		if maxExtendedFunction() >= 0x80000008 {
 			_, _, c, _ := cpuid(0x80000008)
 			return int(c&0xff) + 1
@@ -765,31 +914,101 @@ func (c *CPUInfo) cacheSize() {
 		}
 		_, _, ecx, _ = cpuid(0x80000006)
 		c.Cache.L2 = int(((ecx >> 16) & 0xFFFF) * 1024)
+
+		// CPUID Fn8000_001D_EAX_x[N:0] Cache Properties
+		if maxExtendedFunction() < 0x8000001D {
+			return
+		}
+		for i := uint32(0); i < math.MaxUint32; i++ {
+			eax, ebx, ecx, _ := cpuidex(0x8000001D, i)
+
+			level := (eax >> 5) & 7
+			cacheNumSets := ecx + 1
+			cacheLineSize := 1 + (ebx & 2047)
+			cachePhysPartitions := 1 + ((ebx >> 12) & 511)
+			cacheNumWays := 1 + ((ebx >> 22) & 511)
+
+			typ := eax & 15
+			size := int(cacheNumSets * cacheLineSize * cachePhysPartitions * cacheNumWays)
+			if typ == 0 {
+				return
+			}
+
+			switch level {
+			case 1:
+				switch typ {
+				case 1:
+					// Data cache
+					c.Cache.L1D = size
+				case 2:
+					// Inst cache
+					c.Cache.L1I = size
+				default:
+					if c.Cache.L1D < 0 {
+						c.Cache.L1I = size
+					}
+					if c.Cache.L1I < 0 {
+						c.Cache.L1I = size
+					}
+				}
+			case 2:
+				c.Cache.L2 = size
+			case 3:
+				c.Cache.L3 = size
+			}
+		}
 	}
 
 	return
 }
 
+type SGXEPCSection struct {
+	BaseAddress uint64
+	EPCSize     uint64
+}
+
 type SGXSupport struct {
 	Available           bool
+	LaunchControl       bool
 	SGX1Supported       bool
 	SGX2Supported       bool
 	MaxEnclaveSizeNot64 int64
 	MaxEnclaveSize64    int64
+	EPCSections         []SGXEPCSection
 }
 
-func hasSGX(available bool) (rval SGXSupport) {
+func hasSGX(available, lc bool) (rval SGXSupport) {
 	rval.Available = available
 
 	if !available {
 		return
 	}
 
+	rval.LaunchControl = lc
+
 	a, _, _, d := cpuidex(0x12, 0)
 	rval.SGX1Supported = a&0x01 != 0
 	rval.SGX2Supported = a&0x02 != 0
 	rval.MaxEnclaveSizeNot64 = 1 << (d & 0xFF)     // pow 2
 	rval.MaxEnclaveSize64 = 1 << ((d >> 8) & 0xFF) // pow 2
+	rval.EPCSections = make([]SGXEPCSection, 0)
+
+	for subleaf := uint32(2); subleaf < 2+8; subleaf++ {
+		eax, ebx, ecx, edx := cpuidex(0x12, subleaf)
+		leafType := eax & 0xf
+
+		if leafType == 0 {
+			// Invalid subleaf, stop iterating
+			break
+		} else if leafType == 1 {
+			// EPC Section subleaf
+			baseAddress := uint64(eax&0xfffff000) + (uint64(ebx&0x000fffff) << 32)
+			size := uint64(ecx&0xfffff000) + (uint64(edx&0x000fffff) << 32)
+
+			section := SGXEPCSection{BaseAddress: baseAddress, EPCSize: size}
+			rval.EPCSections = append(rval.EPCSections, section)
+		}
+	}
 
 	return
 }
@@ -820,6 +1039,9 @@ func support() Flags {
 	if (c & 1) != 0 {
 		rval |= SSE3
 	}
+	if (c & (1 << 5)) != 0 {
+		rval |= VMX
+	}
 	if (c & 0x00000200) != 0 {
 		rval |= SSSE3
 	}
@@ -852,7 +1074,11 @@ func support() Flags {
 			rval |= HTT
 		}
 	}
-
+	if vend == AMD && (d&(1<<28)) != 0 && mfi >= 4 {
+		if threadsPerCore() > 1 {
+			rval |= HTT
+		}
+	}
 	// Check XGETBV, OXSAVE and AVX bits
 	if c&(1<<26) != 0 && c&(1<<27) != 0 && c&(1<<28) != 0 {
 		// Check for OS support
@@ -868,6 +1094,7 @@ func support() Flags {
 	// Check AVX2, AVX2 requires OS support, but BMI1/2 don't.
 	if mfi >= 7 {
 		_, ebx, ecx, edx := cpuidex(7, 0)
+		eax1, _, _, _ := cpuidex(7, 1)
 		if (rval&AVX) != 0 && (ebx&0x00000020) != 0 {
 			rval |= AVX2
 		}
@@ -904,6 +1131,9 @@ func support() Flags {
 		if edx&(1<<26) != 0 {
 			rval |= IBPB
 		}
+		if ecx&(1<<30) != 0 {
+			rval |= SGXLC
+		}
 		if edx&(1<<27) != 0 {
 			rval |= STIBP
 		}
@@ -945,6 +1175,35 @@ func support() Flags {
 				if ecx&(1<<1) != 0 {
 					rval |= AVX512VBMI
 				}
+				if ecx&(1<<6) != 0 {
+					rval |= AVX512VBMI2
+				}
+				if ecx&(1<<8) != 0 {
+					rval |= GFNI
+				}
+				if ecx&(1<<9) != 0 {
+					rval |= VAES
+				}
+				if ecx&(1<<10) != 0 {
+					rval |= VPCLMULQDQ
+				}
+				if ecx&(1<<11) != 0 {
+					rval |= AVX512VNNI
+				}
+				if ecx&(1<<12) != 0 {
+					rval |= AVX512BITALG
+				}
+				if ecx&(1<<14) != 0 {
+					rval |= AVX512VPOPCNTDQ
+				}
+				// edx
+				if edx&(1<<8) != 0 {
+					rval |= AVX512VP2INTERSECT
+				}
+				// cpuid eax 07h,ecx=1
+				if eax1&(1<<5) != 0 {
+					rval |= AVX512BF16
+				}
 			}
 		}
 	}
